@@ -25,12 +25,12 @@
 
 #include "basetreeview.h"
 
+#include "algorithm.h"
 #include "progressindicator.h"
+#include "qtcassert.h"
+#include "qtcsettings.h"
 #include "treemodel.h"
 
-#include <utils/algorithm.h>
-#include <utils/qtcassert.h>
-
 #include <QDebug>
 #include <QFontMetrics>
 #include <QHeaderView>
@@ -159,7 +159,7 @@ public:
                 l.append(column);
                 l.append(width);
             }
-            m_settings->setValue(QLatin1String(ColumnKey), l);
+            QtcSettings::setValueWithDefault(m_settings, ColumnKey, l);
             m_settings->endGroup();
         }
     }
@@ -183,7 +183,7 @@ public:
         QAbstractItemModel *m = q->model();
         for (int i = 0; i < 100 && a.isValid(); ++i) {
             const QString s = m->data(a).toString();
-            int w = fm.width(s) + 10;
+            int w = fm.horizontalAdvance(s) + 10;
             if (column == 0) {
                 for (QModelIndex b = a.parent(); b.isValid(); b = b.parent())
                     w += ind;
@@ -204,11 +204,13 @@ public:
         QTC_ASSERT(m, return -1);
 
         QFontMetrics fm = q->fontMetrics();
-        int minimum = fm.width(m->headerData(column, Qt::Horizontal).toString()) + 2 * fm.width(QLatin1Char('m'));
+        int minimum = fm.horizontalAdvance(m->headerData(column, Qt::Horizontal).toString())
+            + 2 * fm.horizontalAdvance(QLatin1Char('m'));
         considerItems(column, q->indexAt(QPoint(1, 1)), &minimum, false);
 
-        QVariant extraIndices = m->data(QModelIndex(), BaseTreeView::ExtraIndicesForColumnWidth);
-        foreach (const QModelIndex &a, extraIndices.value<QModelIndexList>())
+        const QVariant extraIndices = m->data(QModelIndex(), BaseTreeView::ExtraIndicesForColumnWidth);
+        const QList<QModelIndex> values = extraIndices.value<QModelIndexList>();
+        for (const QModelIndex &a : values)
             considerItems(column, a, &minimum, true);
 
         return minimum;
@@ -254,8 +256,8 @@ public:
         // when we have that size already, in that case minimize.
         if (currentSize == suggestedSize) {
             QFontMetrics fm = q->fontMetrics();
-            int headerSize = fm.width(q->model()->headerData(logicalIndex, Qt::Horizontal).toString());
-            int minSize = 10 * fm.width(QLatin1Char('x'));
+            int headerSize = fm.horizontalAdvance(q->model()->headerData(logicalIndex, Qt::Horizontal).toString());
+            int minSize = 10 * fm.horizontalAdvance(QLatin1Char('x'));
             targetSize = qMax(minSize, headerSize);
         }
 
@@ -344,7 +346,7 @@ public:
     QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option,
                           const QModelIndex &index) const override
     {
-        Q_UNUSED(option);
+        Q_UNUSED(option)
         QLabel *label = new QLabel(parent);
         label->setAutoFillBackground(true);
         label->setTextInteractionFlags(Qt::TextSelectableByMouse
@@ -550,6 +552,33 @@ void BaseTreeView::setSpanColumn(int column)
     d->setSpanColumn(column);
 }
 
+void BaseTreeView::enableColumnHiding()
+{
+    header()->setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(header(), &QWidget::customContextMenuRequested, this, [this](const QPoint &pos) {
+        QTC_ASSERT(model(), return);
+        const int columns = model()->columnCount();
+        QMenu menu;
+        int shown = 0;
+        for (int i = 0; i < columns; ++i)
+            shown += !isColumnHidden(i);
+        for (int i = 0; i < columns; ++i) {
+            QString columnName = model()->headerData(i, Qt::Horizontal).toString();
+            QAction *act = menu.addAction(tr("Show %1 Column").arg(columnName));
+            act->setCheckable(true);
+            act->setChecked(!isColumnHidden(i));
+            // Prevent disabling the last visible column as there's no way back.
+            if (shown == 1 && !isColumnHidden(i))
+                act->setEnabled(false);
+            QObject::connect(act, &QAction::toggled, &menu, [this, i](bool on) {
+                setColumnHidden(i, !on);
+            });
+        }
+        menu.addSeparator();
+        menu.exec(mapToGlobal(pos));
+    });
+}
+
 void BaseTreeView::refreshSpanColumn()
 {
     d->rebalanceColumns();
@@ -608,7 +637,7 @@ ItemViewEvent::ItemViewEvent(QEvent *ev, QAbstractItemView *view)
     };
 
     m_sourceModelIndex = fixIndex(m_index);
-    m_selectedRows = Utils::transform(m_selectedRows, fixIndex);
+    m_selectedRows = transform(m_selectedRows, fixIndex);
 }
 
 QModelIndexList ItemViewEvent::currentOrSelectedRows() const
