@@ -79,7 +79,6 @@ public:
     Id id;
     OutputPaneToggleButton *button = nullptr;
     QAction *action = nullptr;
-    bool buttonVisible = false;
 };
 
 static QVector<OutputPaneData> g_outputPanes;
@@ -100,18 +99,15 @@ IOutputPane::IOutputPane(QObject *parent)
 
     m_zoomInButton->setIcon(Utils::Icons::PLUS_TOOLBAR.icon());
     m_zoomInButton->setCommandId(Constants::ZOOM_IN);
-    connect(m_zoomInButton, &QToolButton::clicked, this, [this] { emit zoomIn(1); });
+    connect(m_zoomInButton, &QToolButton::clicked, this, [this] { emit zoomInRequested(1); });
 
     m_zoomOutButton->setIcon(Utils::Icons::MINUS.icon());
     m_zoomOutButton->setCommandId(Constants::ZOOM_OUT);
-    connect(m_zoomOutButton, &QToolButton::clicked, this, [this] { emit zoomOut(1); });
+    connect(m_zoomOutButton, &QToolButton::clicked, this, [this] { emit zoomOutRequested(1); });
 }
 
 IOutputPane::~IOutputPane()
 {
-    if (m_context)
-        ICore::removeContextObject(m_context);
-
     const int i = Utils::indexOf(g_outputPanes, Utils::equal(&OutputPaneData::pane, this));
     QTC_ASSERT(i >= 0, return);
     delete g_outputPanes.at(i).button;
@@ -129,6 +125,10 @@ QList<QWidget *> IOutputPane::toolBarWidgets() const
     return widgets << m_zoomInButton << m_zoomOutButton;
 }
 
+void IOutputPane::visibilityChanged(bool /*visible*/)
+{
+}
+
 void IOutputPane::setFont(const QFont &font)
 {
     emit fontChanged(font);
@@ -198,14 +198,14 @@ void IOutputPane::setupContext(const char *context, QWidget *widget)
 
     const auto zoomInAction = new QAction(this);
     Core::ActionManager::registerAction(zoomInAction, Constants::ZOOM_IN, m_context->context());
-    connect(zoomInAction, &QAction::triggered, this, [this] { emit zoomIn(1); });
+    connect(zoomInAction, &QAction::triggered, this, [this] { emit zoomInRequested(1); });
     const auto zoomOutAction = new QAction(this);
     Core::ActionManager::registerAction(zoomOutAction, Constants::ZOOM_OUT, m_context->context());
-    connect(zoomOutAction, &QAction::triggered, this, [this] { emit zoomOut(1); });
+    connect(zoomOutAction, &QAction::triggered, this, [this] { emit zoomOutRequested(1); });
     const auto resetZoomAction = new QAction(this);
     Core::ActionManager::registerAction(resetZoomAction, Constants::ZOOM_RESET,
                                         m_context->context());
-    connect(resetZoomAction, &QAction::triggered, this, &IOutputPane::resetZoom);
+    connect(resetZoomAction, &QAction::triggered, this, &IOutputPane::resetZoomRequested);
 }
 
 void IOutputPane::setZoomButtonsEnabled(bool enabled)
@@ -303,10 +303,10 @@ void OutputPaneManager::updateMaximizeButton(bool maximized)
 {
     if (maximized) {
         m_instance->m_minMaxAction->setIcon(m_instance->m_minimizeIcon);
-        m_instance->m_minMaxAction->setText(tr("Minimize Output Pane"));
+        m_instance->m_minMaxAction->setText(tr("Minimize"));
     } else {
         m_instance->m_minMaxAction->setIcon(m_instance->m_maximizeIcon);
-        m_instance->m_minMaxAction->setText(tr("Maximize Output Pane"));
+        m_instance->m_minMaxAction->setText(tr("Maximize"));
     }
 }
 
@@ -341,18 +341,18 @@ OutputPaneManager::OutputPaneManager(QWidget *parent) :
     connect(m_clearAction, &QAction::triggered, this, &OutputPaneManager::clearPage);
 
     m_nextAction = new QAction(this);
-    m_nextAction->setIcon(Utils::Icons::NEXT.icon());
+    m_nextAction->setIcon(Utils::Icons::ARROW_DOWN_TOOLBAR.icon());
     m_nextAction->setText(tr("Next Item"));
     connect(m_nextAction, &QAction::triggered, this, &OutputPaneManager::slotNext);
 
     m_prevAction = new QAction(this);
-    m_prevAction->setIcon(Utils::Icons::PREV.icon());
+    m_prevAction->setIcon(Utils::Icons::ARROW_UP_TOOLBAR.icon());
     m_prevAction->setText(tr("Previous Item"));
     connect(m_prevAction, &QAction::triggered, this, &OutputPaneManager::slotPrev);
 
     m_minMaxAction = new QAction(this);
     m_minMaxAction->setIcon(m_maximizeIcon);
-    m_minMaxAction->setText(tr("Maximize Output Pane"));
+    m_minMaxAction->setText(tr("Maximize"));
 
     m_closeButton->setIcon(Icons::CLOSE_SPLIT_BOTTOM.icon());
     connect(m_closeButton, &QAbstractButton::clicked, this, &OutputPaneManager::slotHide);
@@ -391,31 +391,31 @@ OutputPaneManager::OutputPaneManager(QWidget *parent) :
 
     StatusBarManager::addStatusBarWidget(m_buttonsWidget, StatusBarManager::Second);
 
-    ActionContainer *mwindow = ActionManager::actionContainer(Constants::M_WINDOW);
+    ActionContainer *mview = ActionManager::actionContainer(Constants::M_VIEW);
 
     // Window->Output Panes
-    ActionContainer *mpanes = ActionManager::createMenu(Constants::M_WINDOW_PANES);
-    mwindow->addMenu(mpanes, Constants::G_WINDOW_PANES);
-    mpanes->menu()->setTitle(tr("Output &Panes"));
+    ActionContainer *mpanes = ActionManager::createMenu(Constants::M_VIEW_PANES);
+    mview->addMenu(mpanes, Constants::G_VIEW_PANES);
+    mpanes->menu()->setTitle(tr("Out&put"));
     mpanes->appendGroup("Coreplugin.OutputPane.ActionsGroup");
     mpanes->appendGroup("Coreplugin.OutputPane.PanesGroup");
 
     Command *cmd;
 
-    cmd = ActionManager::registerAction(m_clearAction, "Coreplugin.OutputPane.clear");
-    m_clearButton->setDefaultAction(cmd->action());
-    m_clearButton->setIcon(Utils::Icons::CLEAN_TOOLBAR.icon());
+    cmd = ActionManager::registerAction(m_clearAction, Constants::OUTPUTPANE_CLEAR);
+    m_clearButton->setDefaultAction(
+        ProxyAction::proxyActionWithIcon(m_clearAction, Utils::Icons::CLEAN_TOOLBAR.icon()));
     mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
 
     cmd = ActionManager::registerAction(m_prevAction, "Coreplugin.OutputPane.previtem");
     cmd->setDefaultKeySequence(QKeySequence(tr("Shift+F6")));
     m_prevToolButton->setDefaultAction(
-                ProxyAction::proxyActionWithIcon(cmd->action(), Utils::Icons::PREV_TOOLBAR.icon()));
+        ProxyAction::proxyActionWithIcon(m_prevAction, Utils::Icons::ARROW_UP_TOOLBAR.icon()));
     mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
 
     cmd = ActionManager::registerAction(m_nextAction, "Coreplugin.OutputPane.nextitem");
     m_nextToolButton->setDefaultAction(
-                ProxyAction::proxyActionWithIcon(cmd->action(), Utils::Icons::NEXT_TOOLBAR.icon()));
+        ProxyAction::proxyActionWithIcon(m_nextAction, Utils::Icons::ARROW_DOWN_TOOLBAR.icon()));
     cmd->setDefaultKeySequence(QKeySequence(tr("F6")));
     mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
 
@@ -428,8 +428,12 @@ OutputPaneManager::OutputPaneManager(QWidget *parent) :
     m_minMaxButton->setDefaultAction(cmd->action());
 
     mpanes->addSeparator("Coreplugin.OutputPane.ActionsGroup");
+}
 
-    QFontMetrics titleFm = m_titleLabel->fontMetrics();
+void OutputPaneManager::initialize()
+{
+    ActionContainer *mpanes = ActionManager::actionContainer(Constants::M_VIEW_PANES);
+    QFontMetrics titleFm = m_instance->m_titleLabel->fontMetrics();
     int minTitleWidth = 0;
 
     Utils::sort(g_outputPanes, [](const OutputPaneData &d1, const OutputPaneData &d2) {
@@ -442,79 +446,87 @@ OutputPaneManager::OutputPaneManager(QWidget *parent) :
     for (int i = 0; i != n; ++i) {
         OutputPaneData &data = g_outputPanes[i];
         IOutputPane *outPane = data.pane;
-        const int idx = m_outputWidgetPane->addWidget(outPane->outputWidget(this));
+        const int idx = m_instance->m_outputWidgetPane->addWidget(outPane->outputWidget(m_instance));
         QTC_CHECK(idx == i);
 
-        connect(outPane, &IOutputPane::showPage, this, [this, idx](int flags) {
-            showPage(idx, flags);
+        connect(outPane, &IOutputPane::showPage, m_instance, [idx](int flags) {
+            m_instance->showPage(idx, flags);
         });
-        connect(outPane, &IOutputPane::hidePage, this, &OutputPaneManager::slotHide);
+        connect(outPane, &IOutputPane::hidePage, m_instance, &OutputPaneManager::slotHide);
 
-        connect(outPane, &IOutputPane::togglePage, this, [this, idx](int flags) {
-            if (OutputPanePlaceHolder::isCurrentVisible() && currentIndex() == idx)
-                slotHide();
+        connect(outPane, &IOutputPane::togglePage, m_instance, [idx](int flags) {
+            if (OutputPanePlaceHolder::isCurrentVisible() && m_instance->currentIndex() == idx)
+                m_instance->slotHide();
             else
-                showPage(idx, flags);
+                m_instance->showPage(idx, flags);
         });
 
-        connect(outPane, &IOutputPane::navigateStateUpdate, this, [this, idx, outPane] {
-            if (currentIndex() == idx) {
-                m_prevAction->setEnabled(outPane->canNavigate() && outPane->canPrevious());
-                m_nextAction->setEnabled(outPane->canNavigate() && outPane->canNext());
+        connect(outPane, &IOutputPane::navigateStateUpdate, m_instance, [idx, outPane] {
+            if (m_instance->currentIndex() == idx) {
+                m_instance->m_prevAction->setEnabled(outPane->canNavigate()
+                                                     && outPane->canPrevious());
+                m_instance->m_nextAction->setEnabled(outPane->canNavigate() && outPane->canNext());
             }
         });
 
-        QWidget *toolButtonsContainer = new QWidget(m_opToolBarWidgets);
+        QWidget *toolButtonsContainer = new QWidget(m_instance->m_opToolBarWidgets);
         auto toolButtonsLayout = new QHBoxLayout;
         toolButtonsLayout->setContentsMargins(0, 0, 0, 0);
         toolButtonsLayout->setSpacing(0);
-        foreach (QWidget *toolButton, outPane->toolBarWidgets())
+        const QList<QWidget *> toolBarWidgets = outPane->toolBarWidgets();
+        for (QWidget *toolButton : toolBarWidgets)
             toolButtonsLayout->addWidget(toolButton);
         toolButtonsLayout->addStretch(5);
         toolButtonsContainer->setLayout(toolButtonsLayout);
 
-        m_opToolBarWidgets->addWidget(toolButtonsContainer);
+        m_instance->m_opToolBarWidgets->addWidget(toolButtonsContainer);
 
         minTitleWidth = qMax(minTitleWidth, titleFm.horizontalAdvance(outPane->displayName()));
 
         QString suffix = outPane->displayName().simplified();
         suffix.remove(QLatin1Char(' '));
         data.id = baseId.withSuffix(suffix);
-        data.action = new QAction(outPane->displayName(), this);
+        data.action = new QAction(outPane->displayName(), m_instance);
         Command *cmd = ActionManager::registerAction(data.action, data.id);
 
         mpanes->addAction(cmd, "Coreplugin.OutputPane.PanesGroup");
 
         cmd->setDefaultKeySequence(paneShortCut(shortcutNumber));
-        auto button = new OutputPaneToggleButton(shortcutNumber, outPane->displayName(),
+        auto button = new OutputPaneToggleButton(shortcutNumber,
+                                                 outPane->displayName(),
                                                  cmd->action());
         data.button = button;
 
         connect(outPane, &IOutputPane::flashButton, button, [button] { button->flash(); });
-        connect(outPane, &IOutputPane::setBadgeNumber,
-                button, &OutputPaneToggleButton::setIconBadgeNumber);
+        connect(outPane,
+                &IOutputPane::setBadgeNumber,
+                button,
+                &OutputPaneToggleButton::setIconBadgeNumber);
 
         ++shortcutNumber;
-        m_buttonsWidget->layout()->addWidget(data.button);
-        connect(data.button, &QAbstractButton::clicked, this, [this, i] {
-            buttonTriggered(i);
-         });
+        m_instance->m_buttonsWidget->layout()->addWidget(data.button);
+        connect(data.button, &QAbstractButton::clicked, m_instance, [i] {
+            m_instance->buttonTriggered(i);
+        });
 
         bool visible = outPane->priorityInStatusBar() != -1;
         data.button->setVisible(visible);
-        data.buttonVisible = visible;
 
-        connect(data.action, &QAction::triggered, this, [this, i] {
-            shortcutTriggered(i);
+        connect(data.action, &QAction::triggered, m_instance, [i] {
+            m_instance->shortcutTriggered(i);
         });
     }
 
-    m_titleLabel->setMinimumWidth(minTitleWidth + m_titleLabel->contentsMargins().left()
-                                  + m_titleLabel->contentsMargins().right());
-    m_buttonsWidget->layout()->addWidget(m_manageButton);
-    connect(m_manageButton, &QAbstractButton::clicked, this, &OutputPaneManager::popupMenu);
+    m_instance->m_titleLabel->setMinimumWidth(
+        minTitleWidth + m_instance->m_titleLabel->contentsMargins().left()
+        + m_instance->m_titleLabel->contentsMargins().right());
+    m_instance->m_buttonsWidget->layout()->addWidget(m_instance->m_manageButton);
+    connect(m_instance->m_manageButton,
+            &QAbstractButton::clicked,
+            m_instance,
+            &OutputPaneManager::popupMenu);
 
-    readSettings();
+    m_instance->readSettings();
 }
 
 OutputPaneManager::~OutputPaneManager() = default;
@@ -582,7 +594,6 @@ void OutputPaneManager::readSettings()
         if (idx < 0) // happens for e.g. disabled plugins (with outputpanes) that were loaded before
             continue;
         const bool visible = settings->value(QLatin1String(outputPaneVisibleKeyC)).toBool();
-        g_outputPanes[idx].buttonVisible = visible;
         g_outputPanes[idx].button->setVisible(visible);
     }
     settings->endArray();
@@ -694,7 +705,6 @@ void OutputPaneManager::setCurrentIndex(int idx)
         OutputPaneData &data = g_outputPanes[idx];
         IOutputPane *pane = data.pane;
         data.button->show();
-        data.buttonVisible = true;
         pane->visibilityChanged(true);
 
         bool canNavigate = pane->canNavigate();
@@ -714,7 +724,7 @@ void OutputPaneManager::popupMenu()
     for (OutputPaneData &data : g_outputPanes) {
         QAction *act = menu.addAction(data.pane->displayName());
         act->setCheckable(true);
-        act->setChecked(data.buttonVisible);
+        act->setChecked(data.button->isPaneVisible());
         act->setData(idx);
         ++idx;
     }
@@ -724,11 +734,10 @@ void OutputPaneManager::popupMenu()
     idx = result->data().toInt();
     QTC_ASSERT(idx >= 0 && idx < g_outputPanes.size(), return);
     OutputPaneData &data = g_outputPanes[idx];
-    if (data.buttonVisible) {
+    if (data.button->isPaneVisible()) {
         data.pane->visibilityChanged(false);
         data.button->setChecked(false);
         data.button->hide();
-        data.buttonVisible = false;
     } else {
         showPage(idx, IOutputPane::ModeSwitch);
     }
@@ -743,7 +752,7 @@ void OutputPaneManager::saveSettings() const
         const OutputPaneData &data = g_outputPanes.at(i);
         settings->setArrayIndex(i);
         settings->setValue(QLatin1String(outputPaneIdKeyC), data.id.toSetting());
-        settings->setValue(QLatin1String(outputPaneVisibleKeyC), data.buttonVisible);
+        settings->setValue(QLatin1String(outputPaneVisibleKeyC), data.button->isPaneVisible());
     }
     settings->endArray();
     int heightSetting = m_outputPaneHeightSetting;
@@ -788,7 +797,7 @@ OutputPaneToggleButton::OutputPaneToggleButton(int number, const QString &text,
         connect(m_action, &QAction::changed, this, &OutputPaneToggleButton::updateToolTip);
 
     m_flashTimer->setDirection(QTimeLine::Forward);
-    m_flashTimer->setCurveShape(QTimeLine::SineCurve);
+    m_flashTimer->setEasingCurve(QEasingCurve::SineCurve);
     m_flashTimer->setFrameRange(0, 92);
     auto updateSlot = QOverload<>::of(&QWidget::update);
     connect(m_flashTimer, &QTimeLine::valueChanged, this, updateSlot);
@@ -814,7 +823,7 @@ QSize OutputPaneToggleButton::sizeHint() const
     if (!m_badgeNumberLabel.text().isNull())
         s.rwidth() += m_badgeNumberLabel.sizeHint().width() + 1;
 
-    return s.expandedTo(QApplication::globalStrut());
+    return s;
 }
 
 void OutputPaneToggleButton::paintEvent(QPaintEvent*)
@@ -920,6 +929,11 @@ void OutputPaneToggleButton::setIconBadgeNumber(int number)
     updateGeometry();
 }
 
+bool OutputPaneToggleButton::isPaneVisible() const
+{
+    return isVisibleTo(parentWidget());
+}
+
 
 ///////////////////////////////////////////////////////////////////////
 //
@@ -937,7 +951,7 @@ OutputPaneManageButton::OutputPaneManageButton()
 QSize OutputPaneManageButton::sizeHint() const
 {
     ensurePolished();
-    return QSize(numberAreaWidth(), QApplication::globalStrut().height());
+    return QSize(numberAreaWidth(), 16);
 }
 
 void OutputPaneManageButton::paintEvent(QPaintEvent*)
