@@ -226,6 +226,27 @@ using namespace Utils;
 
 static ProgressManagerPrivate *m_instance = nullptr;
 
+const int RASTER = 20;
+
+class StatusDetailsWidgetContainer : public QWidget
+{
+public:
+    StatusDetailsWidgetContainer(QWidget *parent)
+        : QWidget(parent)
+    {}
+
+    QSize sizeHint() const override
+    {
+        // make size fit on raster, to avoid flickering in status bar
+        // because the output pane buttons resize, if the widget changes a lot (like it is the case for
+        // the language server indexing)
+        const QSize preferredSize = layout()->sizeHint();
+        const int preferredWidth = preferredSize.width();
+        const int width = preferredWidth + (RASTER - preferredWidth % RASTER);
+        return {width, preferredSize.height()};
+    }
+};
+
 ProgressManagerPrivate::ProgressManagerPrivate()
     : m_opacityEffect(new QGraphicsOpacityEffect(this))
 {
@@ -269,16 +290,23 @@ void ProgressManagerPrivate::init()
     m_summaryProgressWidget = new QWidget(m_statusBarWidget);
     m_summaryProgressWidget->setVisible(!m_progressViewPinned);
     m_summaryProgressWidget->setGraphicsEffect(m_opacityEffect);
-    m_summaryProgressLayout = new QHBoxLayout(m_summaryProgressWidget);
-    m_summaryProgressLayout->setContentsMargins(0, 0, 0, 2);
-    m_summaryProgressLayout->setSpacing(0);
-    m_summaryProgressWidget->setLayout(m_summaryProgressLayout);
+    auto summaryProgressLayout = new QHBoxLayout(m_summaryProgressWidget);
+    summaryProgressLayout->setContentsMargins(0, 0, 0, 2);
+    summaryProgressLayout->setSpacing(0);
+    m_summaryProgressWidget->setLayout(summaryProgressLayout);
+    auto statusDetailsWidgetContainer = new StatusDetailsWidgetContainer(m_summaryProgressWidget);
+    m_statusDetailsWidgetLayout = new QHBoxLayout(statusDetailsWidgetContainer);
+    m_statusDetailsWidgetLayout->setContentsMargins(0, 0, 0, 0);
+    m_statusDetailsWidgetLayout->setSpacing(0);
+    m_statusDetailsWidgetLayout->addStretch(1);
+    statusDetailsWidgetContainer->setLayout(m_statusDetailsWidgetLayout);
+    summaryProgressLayout->addWidget(statusDetailsWidgetContainer);
     m_summaryProgressBar = new ProgressBar(m_summaryProgressWidget);
     m_summaryProgressBar->setMinimumWidth(70);
     m_summaryProgressBar->setTitleVisible(false);
     m_summaryProgressBar->setSeparatorVisible(false);
     m_summaryProgressBar->setCancelEnabled(false);
-    m_summaryProgressLayout->addWidget(m_summaryProgressBar);
+    summaryProgressLayout->addWidget(m_summaryProgressBar);
     layout->addWidget(m_summaryProgressWidget);
     auto toggleButton = new QToolButton(m_statusBarWidget);
     layout->addWidget(toggleButton);
@@ -503,7 +531,7 @@ void ProgressManagerPrivate::stopFadeOfSummaryProgress()
 
 bool ProgressManagerPrivate::hasError() const
 {
-    foreach (FutureProgress *progress, m_taskList)
+    for (const FutureProgress *progress : qAsConst(m_taskList))
         if (progress->hasError())
             return true;
     return false;
@@ -513,7 +541,7 @@ bool ProgressManagerPrivate::isLastFading() const
 {
     if (m_taskList.isEmpty())
         return false;
-    foreach (FutureProgress *progress, m_taskList) {
+    for (const FutureProgress *progress : qAsConst(m_taskList)) {
         if (!progress->isFading()) // we still have progress bars that are not fading
             return false;
     }
@@ -564,7 +592,7 @@ void ProgressManagerPrivate::removeOneOldTask()
         Id type = (*i)->type();
 
         int taskCount = 0;
-        foreach (FutureProgress *p, m_taskList)
+        for (const FutureProgress *p : qAsConst(m_taskList))
             if (p->type() == type)
                 ++taskCount;
 
@@ -641,11 +669,11 @@ void ProgressManagerPrivate::updateStatusDetailsWidget()
 
     if (m_currentStatusDetailsWidget) {
         m_currentStatusDetailsWidget->hide();
-        m_summaryProgressLayout->removeWidget(m_currentStatusDetailsWidget);
+        m_statusDetailsWidgetLayout->removeWidget(m_currentStatusDetailsWidget);
     }
 
     if (candidateWidget) {
-        m_summaryProgressLayout->insertWidget(0, candidateWidget);
+        m_statusDetailsWidgetLayout->addWidget(candidateWidget);
         candidateWidget->show();
     }
 
