@@ -31,11 +31,13 @@
 
 #include <coreplugin/icore.h>
 #include <coreplugin/shellcommand.h>
+
 #include <utils/hostosinfo.h>
-#include <utils/synchronousprocess.h>
+#include <utils/qtcprocess.h>
 
 #include <QFile>
 #include <QJsonDocument>
+#include <QJsonObject>
 #include <QMessageBox>
 #include <QRegularExpression>
 #include <QSettings>
@@ -155,11 +157,10 @@ bool GerritServer::fillFromRemote(const QString &remote,
     port = r.port;
     user.userName = r.userName.isEmpty() ? parameters.server.user.userName : r.userName;
     if (type == GerritServer::Ssh) {
-        resolveVersion(parameters, forceReload);
-        return true;
+        return resolveVersion(parameters, forceReload);
     }
     curlBinary = parameters.curl;
-    if (curlBinary.isEmpty() || !QFile::exists(curlBinary))
+    if (curlBinary.isEmpty() || !curlBinary.exists())
         return false;
     const StoredHostValidity validity = forceReload ? Invalid : loadSettings();
     switch (validity) {
@@ -170,7 +171,8 @@ bool GerritServer::fillFromRemote(const QString &remote,
         // (can be http://example.net/review)
         ascendPath();
         if (resolveRoot()) {
-            resolveVersion(parameters, forceReload);
+            if (!resolveVersion(parameters, forceReload))
+                return false;
             saveSettings(Valid);
             return true;
         }
@@ -178,8 +180,7 @@ bool GerritServer::fillFromRemote(const QString &remote,
     case NotGerrit:
         return false;
     case Valid:
-        resolveVersion(parameters, false);
-        return true;
+        return resolveVersion(parameters, false);
     }
     return true;
 }
@@ -240,13 +241,13 @@ QStringList GerritServer::curlArguments() const
 
 int GerritServer::testConnection()
 {
-    static GitClient *const client = GitPlugin::client();
+    static GitClient *const client = GitClient::instance();
     const QStringList arguments = curlArguments() << (url(RestUrl) + accountUrlC);
-    const SynchronousProcessResponse resp = client->vcsFullySynchronousExec(
-                QString(), FilePath::fromString(curlBinary), arguments,
-                Core::ShellCommand::NoOutput);
-    if (resp.result == SynchronousProcessResponse::Finished) {
-        QString output = resp.stdOut();
+    QtcProcess proc;
+    client->vcsFullySynchronousExec(proc, {}, {curlBinary, arguments},
+                                    Core::ShellCommand::NoOutput);
+    if (proc.result() == ProcessResult::FinishedWithSuccess) {
+        QString output = proc.cleanedStdOut();
         // Gerrit returns an empty response for /p/qt-creator/a/accounts/self
         // so consider this as 404.
         if (output.isEmpty())
@@ -262,10 +263,10 @@ int GerritServer::testConnection()
         }
         return Success;
     }
-    if (resp.exitCode == CertificateError)
+    if (proc.exitCode() == CertificateError)
         return CertificateError;
     const QRegularExpression errorRegexp("returned error: (\\d+)");
-    QRegularExpressionMatch match = errorRegexp.match(resp.stdErr());
+    QRegularExpressionMatch match = errorRegexp.match(proc.cleanedStdErr());
     if (match.hasMatch())
         return match.captured(1).toInt();
     return UnknownError;
@@ -330,37 +331,37 @@ bool GerritServer::resolveRoot()
     return false;
 }
 
-void GerritServer::resolveVersion(const GerritParameters &p, bool forceReload)
+bool GerritServer::resolveVersion(const GerritParameters &p, bool forceReload)
 {
-    static GitClient *const client = GitPlugin::client();
+    static GitClient *const client = GitClient::instance();
     QSettings *settings = Core::ICore::settings();
     const QString fullVersionKey = "Gerrit/" + host + '/' + versionKey;
     version = settings->value(fullVersionKey).toString();
     if (!version.isEmpty() && !forceReload)
-        return;
+        return true;
     if (type == Ssh) {
-        SynchronousProcess process;
+        QtcProcess proc;
         QStringList arguments;
         if (port)
             arguments << p.portFlag << QString::number(port);
         arguments << hostArgument() << "gerrit" << "version";
-        const SynchronousProcessResponse resp = client->vcsFullySynchronousExec(
-                    QString(), FilePath::fromString(p.ssh), arguments,
-                    Core::ShellCommand::NoOutput);
-        QString stdOut = resp.stdOut().trimmed();
+        client->vcsFullySynchronousExec(proc, {}, {p.ssh, arguments}, Core::ShellCommand::NoOutput);
+        QString stdOut = proc.cleanedStdOut().trimmed();
         stdOut.remove("gerrit version ");
         version = stdOut;
+        if (version.isEmpty())
+            return false;
     } else {
         const QStringList arguments = curlArguments() << (url(RestUrl) + versionUrlC);
-        const SynchronousProcessResponse resp = client->vcsFullySynchronousExec(
-                    QString(), FilePath::fromString(curlBinary), arguments,
-                    Core::ShellCommand::NoOutput);
+        QtcProcess proc;
+        client->vcsFullySynchronousExec(proc, {}, {curlBinary, arguments},
+                                        Core::ShellCommand::NoOutput);
         // REST endpoint for version is only available from 2.8 and up. Do not consider invalid
         // if it fails.
-        if (resp.result == SynchronousProcessResponse::Finished) {
-            QString output = resp.stdOut();
+        if (proc.result() == ProcessResult::FinishedWithSuccess) {
+            QString output = proc.cleanedStdOut();
             if (output.isEmpty())
-                return;
+                return false;
             output.remove(0, output.indexOf('\n')); // Strip first line
             output.remove('\n');
             output.remove('"');
@@ -368,6 +369,7 @@ void GerritServer::resolveVersion(const GerritParameters &p, bool forceReload)
         }
     }
     settings->setValue(fullVersionKey, version);
+    return true;
 }
 
 } // namespace Internal
