@@ -41,6 +41,7 @@
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtversionmanager.h>
 
+#include <utils/algorithm.h>
 #include <utils/aspects.h>
 #include <utils/layoutbuilder.h>
 #include <utils/macroexpander.h>
@@ -88,7 +89,7 @@ public:
     bool isManagedByTarget() const { return m_isManagedByTarget; }
 
 private:
-    void setVisibleDynamic(bool visible) final;
+    void setVisibleDynamic(bool visible);
 
     const ProjectExplorer::Kit *m_kit = nullptr;
     QMap<QString, QString> m_abisToArchMap;
@@ -109,7 +110,7 @@ void ArchitecturesAspect::addToLayout(LayoutBuilder &builder)
 {
     MultiSelectionAspect::addToLayout(builder);
     const auto changeHandler = [this] {
-        const BaseQtVersion *qtVersion = QtKitAspect::qtVersion(m_kit);
+        const QtVersion *qtVersion = QtKitAspect::qtVersion(m_kit);
         if (!qtVersion) {
             setVisibleDynamic(false);
             return;
@@ -129,8 +130,8 @@ void ArchitecturesAspect::addToLayout(LayoutBuilder &builder)
 
         setVisibleDynamic(true);
     };
-    connect(KitManager::instance(), &KitManager::kitsChanged, builder.layout(), changeHandler);
-    connect(this, &ArchitecturesAspect::changed, builder.layout(), changeHandler);
+    connect(KitManager::instance(), &KitManager::kitsChanged, this, changeHandler);
+    connect(this, &ArchitecturesAspect::changed, this, changeHandler);
     changeHandler();
 }
 
@@ -146,7 +147,7 @@ QStringList ArchitecturesAspect::selectedArchitectures() const
 
 void ArchitecturesAspect::setVisibleDynamic(bool visible)
 {
-    MultiSelectionAspect::setVisibleDynamic(visible);
+    MultiSelectionAspect::setVisible(visible);
     m_isManagedByTarget = visible;
 }
 
@@ -497,7 +498,7 @@ void QbsBuildStep::handleProcessResult(
     if (success && !hasOutput)
         return;
 
-    emit addOutput(executable.toUserOutput() + ' '  + QtcProcess::joinArgs(arguments),
+    emit addOutput(executable.toUserOutput() + ' '  + ProcessArgs::joinArgs(arguments),
                    OutputFormat::Stdout);
     for (const QString &line : stdErr)
         emit addOutput(line, OutputFormat::Stderr);
@@ -668,7 +669,7 @@ QbsBuildStepConfigWidget::QbsBuildStepConfigWidget(QbsBuildStep *step) :
     installDirChooser = new PathChooser(this);
     installDirChooser->setExpectedKind(PathChooser::Directory);
 
-    LayoutBuilder builder(this);
+    Layouting::Form builder;
     builder.addRow(m_qbsStep->m_buildVariant);
     builder.addRow(m_qbsStep->m_selectedAbis);
     builder.addRow(m_qbsStep->m_maxJobCount);
@@ -686,6 +687,7 @@ QbsBuildStepConfigWidget::QbsBuildStepConfigWidget(QbsBuildStep *step) :
 
     builder.addRow({tr("Installation directory:"), installDirChooser});
     builder.addRow(m_qbsStep->m_commandLine);
+    builder.attachTo(this, false);
 
     propertyEdit->setToolTip(tr("Properties to pass to the project."));
     defaultInstallDirCheckBox->setText(tr("Use default location"));
@@ -773,7 +775,7 @@ void QbsBuildStepConfigWidget::updatePropertyEdit(const QVariantMap &data)
     for (QVariantMap::const_iterator i = editable.constBegin(); i != editable.constEnd(); ++i)
         propertyList.append(i.key() + ':' + i.value().toString());
 
-    propertyEdit->setText(QtcProcess::joinArgs(propertyList));
+    propertyEdit->setText(ProcessArgs::joinArgs(propertyList));
 }
 
 void QbsBuildStep::changeBuildVariant()
@@ -850,10 +852,9 @@ QbsBuildStep *QbsBuildStepConfigWidget::qbsStep() const
 
 bool QbsBuildStepConfigWidget::validateProperties(Utils::FancyLineEdit *edit, QString *errorMessage)
 {
-    Utils::QtcProcess::SplitError err;
-    QStringList argList = Utils::QtcProcess::splitArgs(edit->text(), Utils::HostOsInfo::hostOs(),
-                                                       false, &err);
-    if (err != Utils::QtcProcess::SplitOk) {
+    ProcessArgs::SplitError err;
+    QStringList argList = ProcessArgs::splitArgs(edit->text(), HostOsInfo::hostOs(), false, &err);
+    if (err != ProcessArgs::SplitOk) {
         if (errorMessage)
             *errorMessage = tr("Could not split properties.");
         return false;
