@@ -25,6 +25,8 @@
 
 #include "designeractionmanagerview.h"
 
+#include <customnotifications.h>
+
 #include <selectioncontext.h>
 #include <actioninterface.h>
 #include <variantproperty.h>
@@ -32,7 +34,7 @@
 namespace QmlDesigner {
 
 DesignerActionManagerView::DesignerActionManagerView()
-    : AbstractView(0),
+    : AbstractView(nullptr),
       m_designerActionManager(this),
       m_isInRewriterTransaction(false),
       m_setupContextDirty(false)
@@ -53,7 +55,7 @@ void DesignerActionManagerView::modelAboutToBeDetached(Model *model)
 
 void DesignerActionManagerView::nodeCreated(const ModelNode &)
 {
-    setupContext(SelectionContext::UpdateMode::Fast);
+    setupContext(SelectionContext::UpdateMode::NodeCreated);
 }
 
 void DesignerActionManagerView::nodeRemoved(const ModelNode &, const NodeAbstractProperty &, AbstractView::PropertyChangeFlags)
@@ -63,17 +65,17 @@ void DesignerActionManagerView::nodeRemoved(const ModelNode &, const NodeAbstrac
 
 void DesignerActionManagerView::nodeAboutToBeReparented(const ModelNode &, const NodeAbstractProperty &, const NodeAbstractProperty &, AbstractView::PropertyChangeFlags)
 {
-    setupContext(SelectionContext::UpdateMode::Fast);
+    setupContext(SelectionContext::UpdateMode::NodeHierachy);
 }
 
 void DesignerActionManagerView::nodeReparented(const ModelNode &, const NodeAbstractProperty &, const NodeAbstractProperty &, AbstractView::PropertyChangeFlags)
 {
-    setupContext(SelectionContext::UpdateMode::Fast);
+    setupContext(SelectionContext::UpdateMode::NodeHierachy);
 }
 
 void DesignerActionManagerView::propertiesRemoved(const QList<AbstractProperty> &)
 {
-    setupContext(SelectionContext::UpdateMode::Fast);
+    setupContext(SelectionContext::UpdateMode::Properties);
 }
 
 void DesignerActionManagerView::rootNodeTypeChanged(const QString &, int, int)
@@ -101,7 +103,7 @@ void DesignerActionManagerView::currentStateChanged(const ModelNode &)
 
 void DesignerActionManagerView::selectedNodesChanged(const QList<ModelNode> &selectedNodes, const QList<ModelNode> &)
 {
-    setupContext();
+    setupContext(SelectionContext::UpdateMode::Fast);
 
     /* This breaks encapsulation, but the selection state is a very minor information.
      * Without this signal the ShortcutManager would have to be refactored completely.
@@ -110,9 +112,9 @@ void DesignerActionManagerView::selectedNodesChanged(const QList<ModelNode> &sel
     emit selectionChanged(!selectedNodes.isEmpty(), singleSelectedModelNode().isRootNode());
 }
 
-void DesignerActionManagerView::nodeOrderChanged(const NodeListProperty &, const ModelNode &, int)
+void DesignerActionManagerView::nodeOrderChanged(const NodeListProperty &)
 {
-    setupContext(SelectionContext::UpdateMode::Fast);
+    setupContext(SelectionContext::UpdateMode::NodeHierachy);
 }
 
 void DesignerActionManagerView::importsChanged(const QList<Import> &, const QList<Import> &)
@@ -122,27 +124,32 @@ void DesignerActionManagerView::importsChanged(const QList<Import> &, const QLis
 
 void DesignerActionManagerView::signalHandlerPropertiesChanged(const QVector<SignalHandlerProperty> &, AbstractView::PropertyChangeFlags)
 {
-    setupContext(SelectionContext::UpdateMode::Fast);
+    setupContext(SelectionContext::UpdateMode::Properties);
 }
 
 void DesignerActionManagerView::variantPropertiesChanged(const QList<VariantProperty> &, AbstractView::PropertyChangeFlags propertyChangeFlag)
 {
     if (propertyChangeFlag == AbstractView::PropertiesAdded)
-        setupContext(SelectionContext::UpdateMode::Fast);
+        setupContext(SelectionContext::UpdateMode::Properties);
     else if (hasSingleSelectedModelNode())
-        setupContext(SelectionContext::UpdateMode::Fast);
+        setupContext(SelectionContext::UpdateMode::Properties);
 }
 
 void DesignerActionManagerView::bindingPropertiesChanged(const QList<BindingProperty> &, AbstractView::PropertyChangeFlags propertyChangeFlag)
 {
     if (propertyChangeFlag == AbstractView::PropertiesAdded)
-        setupContext(SelectionContext::UpdateMode::Fast);
+        setupContext(SelectionContext::UpdateMode::Properties);
 }
 
-void DesignerActionManagerView::instancePropertyChanged(const QList<QPair<ModelNode, PropertyName> > &)
+void DesignerActionManagerView::customNotification(const AbstractView * /*view*/,
+                                      const QString &identifier,
+                                      const QList<ModelNode> & /* nodeList */,
+                                      const QList<QVariant> & /*data */)
 {
-    if (hasSingleSelectedModelNode())
-        setupContext(SelectionContext::UpdateMode::Fast);
+    if (identifier == StartRewriterAmend)
+        m_isInRewriterTransaction = true;
+    else if (identifier == EndRewriterAmend)
+        m_isInRewriterTransaction = false;
 }
 
 DesignerActionManager &DesignerActionManagerView::designerActionManager()
@@ -171,7 +178,8 @@ void DesignerActionManagerView::setupContext(SelectionContext::UpdateMode update
     }
     SelectionContext selectionContext(this);
     selectionContext.setUpdateMode(updateMode);
-    foreach (ActionInterface* action, m_designerActionManager.designerActions()) {
+    const QList<ActionInterface *> actions = m_designerActionManager.designerActions();
+    for (ActionInterface *action : actions) {
         action->currentContextChanged(selectionContext);
     }
     m_setupContextDirty = false;
