@@ -29,19 +29,16 @@
 
 #include <QPointer>
 
-
 namespace QmlDesigner {
 
 class ItemLibraryWidget;
-class ImageCacheData;
-class AsynchronousImageCache;
 
 class ItemLibraryView : public AbstractView
 {
     Q_OBJECT
 
 public:
-    ItemLibraryView(QObject* parent = nullptr);
+    ItemLibraryView(class AsynchronousImageCache &imageCache);
     ~ItemLibraryView() override;
 
     bool hasWidget() const override;
@@ -58,15 +55,11 @@ public:
     void customNotification(const AbstractView *view, const QString &identifier,
                             const QList<ModelNode> &nodeList, const QList<QVariant> &data) override;
 
-    void setResourcePath(const QString &resourcePath);
-
-    AsynchronousImageCache &imageCache();
-
 protected:
     void updateImports();
 
 private:
-    std::unique_ptr<ImageCacheData> m_imageCacheData;
+    AsynchronousImageCache &m_imageCache;
     QPointer<ItemLibraryWidget> m_widget;
     bool m_hasErrors = false;
     QVariantMap m_importableExtensions3DMap;
