@@ -159,7 +159,7 @@ bool InternalNode::hasId() const
 }
 
 
-uint qHash(const InternalNodePointer& node)
+Utils::QHashValueType qHash(const InternalNodePointer& node)
 {
     if (node.isNull())
         return ::qHash(-1);
@@ -187,7 +187,7 @@ bool InternalNode::hasAuxiliaryData(const PropertyName &name) const
     return m_auxiliaryDataHash.contains(name);
 }
 
-QHash<PropertyName, QVariant> InternalNode::auxiliaryData() const
+const QHash<PropertyName, QVariant> &InternalNode::auxiliaryData() const
 {
     return m_auxiliaryDataHash;
 }
@@ -311,7 +311,8 @@ QList<InternalProperty::Pointer> InternalNode::propertyList() const
 QList<InternalNodeAbstractProperty::Pointer> InternalNode::nodeAbstractPropertyList() const
 {
     QList<InternalNodeAbstractProperty::Pointer> abstractPropertyList;
-    foreach (const InternalProperty::Pointer &property, propertyList()) {
+    const QList<InternalProperty::Pointer> properties = propertyList();
+    for (const InternalProperty::Pointer &property : properties) {
         if (property->isNodeAbstractProperty())
             abstractPropertyList.append(property->toNodeAbstractProperty());
     }
@@ -323,7 +324,8 @@ QList<InternalNodeAbstractProperty::Pointer> InternalNode::nodeAbstractPropertyL
 QList<InternalNode::Pointer> InternalNode::allSubNodes() const
 {
     QList<InternalNode::Pointer> nodeList;
-    foreach (const InternalNodeAbstractProperty::Pointer &property, nodeAbstractPropertyList()) {
+    const QList<InternalNodeAbstractProperty::Pointer> properties = nodeAbstractPropertyList();
+    for (const InternalNodeAbstractProperty::Pointer &property : properties) {
         nodeList.append(property->allSubNodes());
     }
 
@@ -333,7 +335,8 @@ QList<InternalNode::Pointer> InternalNode::allSubNodes() const
 QList<InternalNode::Pointer> InternalNode::allDirectSubNodes() const
 {
     QList<InternalNode::Pointer> nodeList;
-    foreach (const InternalNodeAbstractProperty::Pointer &property, nodeAbstractPropertyList()) {
+    const QList<InternalNodeAbstractProperty::Pointer> properties = nodeAbstractPropertyList();
+    for (const InternalNodeAbstractProperty::Pointer &property : properties) {
         nodeList.append(property->directSubNodes());
     }
 
@@ -386,5 +389,15 @@ void InternalNode::setNodeSourceType(int i)
     m_nodeSourceType = i;
 }
 
+QString InternalNode::behaviorPropertyName() const
+{
+    return m_behaviorPropertyName;
+}
+
+void InternalNode::setBehaviorPropertyName(const QString &name)
+{
+    m_behaviorPropertyName = name;
+}
+
 }
 }
