@@ -67,6 +67,7 @@
 
 #include <utils/algorithm.h>
 
+#include <QDrag>
 #include <QRegularExpression>
 
 /*!
@@ -96,9 +97,11 @@ ModelPrivate::ModelPrivate(Model *model)
                                     0,
                                     PropertyListType(),
                                     PropertyListType(),
-                                    QString(),
+                                    {},
                                     ModelNode::NodeWithoutSource,
+                                    {},
                                     true);
+
     m_currentStateNode = m_rootInternalNode;
     m_currentTimelineNode = m_rootInternalNode;
 }
@@ -249,6 +252,7 @@ InternalNodePointer ModelPrivate::createNode(const TypeName &typeName,
                                              const QList<QPair<PropertyName, QVariant>> &auxPropertyList,
                                              const QString &nodeSource,
                                              ModelNode::NodeSourceType nodeSourceType,
+                                             const QString &behaviorPropertyName,
                                              bool isRootNode)
 {
     if (typeName.isEmpty())
@@ -262,6 +266,8 @@ InternalNodePointer ModelPrivate::createNode(const TypeName &typeName,
     InternalNodePointer newNode = InternalNode::create(typeName, majorVersion, minorVersion, internalId);
     newNode->setNodeSourceType(nodeSourceType);
 
+    newNode->setBehaviorPropertyName(behaviorPropertyName);
+
     using PropertyPair = QPair<PropertyName, QVariant>;
 
     for (const PropertyPair &propertyPair : propertyList) {
@@ -588,6 +594,21 @@ void ModelPrivate::notifyImport3DSupportChanged(const QVariantMap &supportMap)
     notifyInstanceChanges([&](AbstractView *view) { view->updateImport3DSupport(supportMap); });
 }
 
+void ModelPrivate::notifyModelAtPosResult(const ModelNode &modelNode)
+{
+    notifyInstanceChanges([&](AbstractView *view) { view->modelAtPosReady(modelNode); });
+}
+
+void ModelPrivate::notifyDragStarted(QMimeData *mimeData)
+{
+    notifyInstanceChanges([&](AbstractView *view) { view->dragStarted(mimeData); });
+}
+
+void ModelPrivate::notifyDragEnded()
+{
+    notifyInstanceChanges([&](AbstractView *view) { view->dragEnded(); });
+}
+
 void ModelPrivate::notifyRewriterBeginTransaction()
 {
     notifyNodeInstanceViewLast([&](AbstractView *view) { view->rewriterBeginTransaction(); });
@@ -1379,14 +1400,18 @@ void Model::changeImports(const QList<Import> &importsToBeAdded,
 
 void Model::setPossibleImports(const QList<Import> &possibleImports)
 {
-    d->m_possibleImportList = possibleImports;
-    d->notifyPossibleImportsChanged(possibleImports);
+    if (d->m_possibleImportList != possibleImports) {
+        d->m_possibleImportList = possibleImports;
+        d->notifyPossibleImportsChanged(possibleImports);
+    }
 }
 
 void Model::setUsedImports(const QList<Import> &usedImports)
 {
-    d->m_usedImportList = usedImports;
-    d->notifyUsedImportsChanged(usedImports);
+    if (d->m_usedImportList != usedImports) {
+        d->m_usedImportList = usedImports;
+        d->notifyUsedImportsChanged(usedImports);
+    }
 }
 
 static bool compareVersions(const QString &version1, const QString &version2, bool allowHigherVersion)
@@ -1451,6 +1476,13 @@ bool Model::hasId(const QString &id) const
     return d->hasId(id);
 }
 
+bool Model::hasImport(const QString &importUrl) const
+{
+    return Utils::anyOf(imports(), [&](const Import &import) {
+        return import.url() == importUrl;
+    });
+}
+
 static QString firstCharToLower(const QString &string)
 {
     QString resultString = string;
@@ -1492,6 +1524,24 @@ QString Model::generateNewId(const QString &prefixName, const QString &fallbackP
     return newId;
 }
 
+void Model::startDrag(QMimeData *mimeData, const QPixmap &icon)
+{
+    d->notifyDragStarted(mimeData);
+
+    auto drag = new QDrag(this);
+    drag->setPixmap(icon);
+    drag->setMimeData(mimeData);
+    if (drag->exec() == Qt::IgnoreAction)
+        endDrag();
+
+    drag->deleteLater();
+}
+
+void Model::endDrag()
+{
+    d->notifyDragEnded();
+}
+
 QString Model::generateNewId(const QString &prefixName) const
 {
     return generateNewId(prefixName, QStringLiteral("element"));
