@@ -25,7 +25,6 @@
 
 #include "modelnode.h"
 #include <abstractproperty.h>
-#include <abstractview.h>
 #include <model.h>
 #include <nodemetainfo.h>
 #include "internalnode_p.h"
@@ -144,7 +143,7 @@ QString ModelNode::id() const
 QString ModelNode::validId()
 {
     if (id().isEmpty())
-        setIdWithRefactoring(view()->generateNewId(simplifiedTypeName()));
+        setIdWithRefactoring(model()->generateNewId(simplifiedTypeName()));
 
     return id();
 }
@@ -195,7 +194,11 @@ static bool isIdToAvoid(const QString& id)
         "enabled",
         "anchors",
         "texture",
-        "shaderInfo"
+        "shaderInfo",
+        "sprite",
+        "spriteSequence",
+        "baseState",
+        "rect"
     };
 
     return ids.contains(id);
@@ -203,7 +206,7 @@ static bool isIdToAvoid(const QString& id)
 
 static bool idContainsWrongLetter(const QString& id)
 {
-    static QRegularExpression idExpr(QStringLiteral("^[a-z_][a-zA-Z0-9_]+$"));
+    static QRegularExpression idExpr(QStringLiteral("^[a-z_][a-zA-Z0-9_]*$"));
     return !id.contains(idExpr);
 }
 
@@ -212,6 +215,29 @@ bool ModelNode::isValidId(const QString &id)
     return id.isEmpty() || (!idContainsWrongLetter(id) && !idIsQmlKeyWord(id) && !isIdToAvoid(id));
 }
 
+QString ModelNode::getIdValidityErrorMessage(const QString &id)
+{
+    if (isValidId(id))
+        return {}; // valid
+
+    if (id.at(0).isUpper())
+        return QObject::tr("ID cannot start with an uppercase character (%1).").arg(id);
+
+    if (id.at(0).isDigit())
+        return QObject::tr("ID cannot start with a number (%1).").arg(id);
+
+    if (id.contains(' '))
+        return QObject::tr("ID cannot include whitespace (%1).").arg(id);
+
+    if (idIsQmlKeyWord(id))
+        return QObject::tr("%1 is a reserved QML keyword.").arg(id);
+
+    if (isIdToAvoid(id))
+        return QObject::tr("%1 is a reserved property keyword.").arg(id);
+
+    return QObject::tr("ID includes invalid characters (%1).").arg(id);
+}
+
 bool ModelNode::hasId() const
 {
     if (!isValid())
@@ -245,7 +271,8 @@ void ModelNode::setIdWithoutRefactoring(const QString &id)
     if (id == m_internalNode->id())
         return;
 
-    if (view()->hasId(id))
+
+    if (model()->hasId(id))
         throw InvalidIdException(__LINE__, __FUNCTION__, __FILE__, id.toUtf8(), InvalidIdException::DuplicateId);
 
     m_model.data()->d->changeNodeId(internalNode(), id);
@@ -569,7 +596,8 @@ QList<AbstractProperty> ModelNode::properties() const
 
     QList<AbstractProperty> propertyList;
 
-    foreach (const PropertyName &propertyName, internalNode()->propertyNameList()) {
+    const QList<PropertyName> propertyNames = internalNode()->propertyNameList();
+    for (const PropertyName &propertyName : propertyNames) {
         AbstractProperty property(propertyName, internalNode(), model(), view());
         propertyList.append(property);
     }
@@ -588,7 +616,8 @@ QList<VariantProperty> ModelNode::variantProperties() const
 {
     QList<VariantProperty> propertyList;
 
-    foreach (const AbstractProperty &abstractProperty, properties())
+    const QList<AbstractProperty> abstractProperties = properties();
+    for (const AbstractProperty &abstractProperty : abstractProperties)
         if (abstractProperty.isVariantProperty())
             propertyList.append(abstractProperty.toVariantProperty());
     return propertyList;
@@ -598,7 +627,8 @@ QList<NodeAbstractProperty> ModelNode::nodeAbstractProperties() const
 {
     QList<NodeAbstractProperty> propertyList;
 
-    foreach (const AbstractProperty &nodeAbstractProperty, properties())
+    const QList<AbstractProperty> abstractProperties = properties();
+    for (const AbstractProperty &nodeAbstractProperty : abstractProperties)
         if (nodeAbstractProperty.isNodeAbstractProperty())
             propertyList.append(nodeAbstractProperty.toNodeAbstractProperty());
     return propertyList;
@@ -608,7 +638,8 @@ QList<NodeProperty> ModelNode::nodeProperties() const
 {
     QList<NodeProperty> propertyList;
 
-    foreach (const AbstractProperty &nodeProperty, properties())
+    const QList<AbstractProperty> abstractProperties = properties();
+    for (const AbstractProperty &nodeProperty : abstractProperties)
         if (nodeProperty.isNodeProperty())
             propertyList.append(nodeProperty.toNodeProperty());
     return propertyList;
@@ -618,7 +649,8 @@ QList<NodeListProperty> ModelNode::nodeListProperties() const
 {
     QList<NodeListProperty> propertyList;
 
-    foreach (const AbstractProperty &nodeListProperty, properties())
+    const QList<AbstractProperty> abstractProperties = properties();
+    for (const AbstractProperty &nodeListProperty : abstractProperties)
         if (nodeListProperty.isNodeListProperty())
             propertyList.append(nodeListProperty.toNodeListProperty());
     return propertyList;
@@ -635,7 +667,8 @@ QList<BindingProperty> ModelNode::bindingProperties() const
 {
     QList<BindingProperty> propertyList;
 
-    foreach (const AbstractProperty &bindingProperty, properties())
+    const QList<AbstractProperty> abstractProperties = properties();
+    for (const AbstractProperty &bindingProperty : abstractProperties)
         if (bindingProperty.isBindingProperty())
             propertyList.append(bindingProperty.toBindingProperty());
     return propertyList;
@@ -645,7 +678,8 @@ QList<SignalHandlerProperty> ModelNode::signalProperties() const
 {
     QList<SignalHandlerProperty> propertyList;
 
-    foreach (const AbstractProperty &property, properties())
+    const QList<AbstractProperty> abstractProperties = properties();
+    for (const AbstractProperty &property : abstractProperties)
         if (property.isSignalHandlerProperty())
             propertyList.append(property.toSignalHandlerProperty());
     return propertyList;
@@ -745,12 +779,6 @@ Internal::InternalNodePointer ModelNode::internalNode() const
     return m_internalNode;
 }
 
-
-uint qHash(const ModelNode &node)
-{
-    return ::qHash(node.internalId());
-}
-
 /*!
 \brief returns the model of the node
 \return returns the model of the node
@@ -1028,6 +1056,11 @@ void ModelNode::setAuxiliaryData(const PropertyName &name, const QVariant &data)
     m_model.data()->d->setAuxiliaryData(internalNode(), name, data);
 }
 
+void ModelNode::setAuxiliaryDataWithoutLock(const PropertyName &name, const QVariant &data) const
+{
+     m_model.data()->d->setAuxiliaryData(internalNode(), name, data);
+}
+
 void ModelNode::removeAuxiliaryData(const PropertyName &name) const
 {
     Internal::WriteLocker locker(m_model.data());
@@ -1278,6 +1311,22 @@ void ModelNode::setNodeSource(const QString &newNodeSource)
     m_model.data()->d->setNodeSource(internalNode(), newNodeSource);
 }
 
+void ModelNode::setNodeSource(const QString &newNodeSource, NodeSourceType type)
+{
+    Internal::WriteLocker locker(m_model.data());
+
+    if (!isValid()) {
+        Q_ASSERT_X(isValid(), Q_FUNC_INFO, "model node is invalid");
+        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
+    }
+
+    if (internalNode()->nodeSourceType() == type && internalNode()->nodeSource() == newNodeSource)
+        return;
+
+    internalNode()->setNodeSourceType(type); // Set type first as it doesn't trigger any notifies
+    m_model.data()->d->setNodeSource(internalNode(), newNodeSource);
+}
+
 QString ModelNode::nodeSource() const
 {
     if (!isValid())
@@ -1387,4 +1436,12 @@ QIcon ModelNode::typeIcon() const
     return QIcon(QStringLiteral(":/ItemLibrary/images/item-invalid-icon.png"));
 }
 
+QString ModelNode::behaviorPropertyName() const
+{
+    if (m_internalNode.isNull())
+        return {};
+
+    return m_internalNode->behaviorPropertyName();
+}
+
 }
