@@ -53,7 +53,7 @@ Q_DECLARE_METATYPE(QmlPreview::QmlPreviewRunControlList)
 namespace QmlDesigner {
 static QObject *s_previewPlugin = nullptr;
 
-QmlPreviewPlugin::QmlPreviewPlugin()
+QmlPreviewWidgetPlugin::QmlPreviewWidgetPlugin()
 {
     DesignerActionManager &designerActionManager =
             QmlDesignerPlugin::instance()->designerActionManager();
@@ -84,18 +84,20 @@ QmlPreviewPlugin::QmlPreviewPlugin()
     if (s_previewPlugin) {
         auto fpsAction = new FpsAction;
         designerActionManager.addDesignerAction(fpsAction);
-        s_previewPlugin->setProperty("fpsHandler", QVariant::fromValue<QmlPreview::QmlPreviewFpsHandler>(FpsLabelAction::fpsHandler));
+        bool hasFpsHandler =
+            s_previewPlugin->setProperty("fpsHandler", QVariant::fromValue<QmlPreview::QmlPreviewFpsHandler>(FpsLabelAction::fpsHandler));
+        QTC_CHECK(hasFpsHandler);
         auto switchLanguageAction = new SwitchLanguageAction;
         designerActionManager.addDesignerAction(switchLanguageAction);
     }
 }
 
-QString QmlPreviewPlugin::pluginName() const
+QString QmlPreviewWidgetPlugin::pluginName() const
 {
     return QLatin1String("QmlPreviewPlugin");
 }
 
-void QmlPreviewPlugin::stopAllRunControls()
+void QmlPreviewWidgetPlugin::stopAllRunControls()
 {
     QTC_ASSERT(s_previewPlugin, return);
 
@@ -107,7 +109,7 @@ void QmlPreviewPlugin::stopAllRunControls()
 
 }
 
-void QmlPreviewPlugin::handleRunningPreviews()
+void QmlPreviewWidgetPlugin::handleRunningPreviews()
 {
     QTC_ASSERT(s_previewPlugin, return);
 
@@ -122,21 +124,23 @@ void QmlPreviewPlugin::handleRunningPreviews()
     }
 }
 
-QString QmlPreviewPlugin::metaInfo() const
+QString QmlPreviewWidgetPlugin::metaInfo() const
 {
     return QLatin1String(":/qmlpreviewplugin/qmlpreview.metainfo");
 }
 
-void QmlPreviewPlugin::setQmlFile()
+void QmlPreviewWidgetPlugin::setQmlFile()
 {
     if (s_previewPlugin) {
         const Utils::FilePath qmlFileName =
                 QmlDesignerPlugin::instance()->currentDesignDocument()->fileName();
-        s_previewPlugin->setProperty("previewedFile", qmlFileName.toString());
+        bool hasPreviewedFile =
+            s_previewPlugin->setProperty("previewedFile", qmlFileName.toString());
+        QTC_CHECK(hasPreviewedFile);
     }
 }
 
-float QmlPreviewPlugin::zoomFactor()
+float QmlPreviewWidgetPlugin::zoomFactor()
 {
     QVariant zoomFactorVariant = 1.0;
     if (s_previewPlugin && !s_previewPlugin->property("zoomFactor").isNull())
@@ -144,27 +148,31 @@ float QmlPreviewPlugin::zoomFactor()
     return zoomFactorVariant.toFloat();
 }
 
-void QmlPreviewPlugin::setZoomFactor(float zoomFactor)
+void QmlPreviewWidgetPlugin::setZoomFactor(float zoomFactor)
 {
-    if (s_previewPlugin)
-        s_previewPlugin->setProperty("zoomFactor", zoomFactor);
+    if (auto s_previewPlugin = getPreviewPlugin()) {
+        bool hasZoomFactor = s_previewPlugin->setProperty("zoomFactor", zoomFactor);
+        QTC_CHECK(hasZoomFactor);
+    }
 }
 
-void QmlPreviewPlugin::setLanguageLocale(const QString &locale)
+void QmlPreviewWidgetPlugin::setLanguageLocale(const QString &locale)
 {
-    if (s_previewPlugin)
-        s_previewPlugin->setProperty("locale", locale);
+    if (auto s_previewPlugin = getPreviewPlugin()) {
+        bool hasLocaleIsoCode = s_previewPlugin->setProperty("localeIsoCode", locale);
+        QTC_CHECK(hasLocaleIsoCode);
+    }
 }
 
-QObject *QmlPreviewPlugin::getPreviewPlugin()
+QObject *QmlPreviewWidgetPlugin::getPreviewPlugin()
 {
-    auto pluginIt = std::find_if(ExtensionSystem::PluginManager::plugins().begin(),
-                                 ExtensionSystem::PluginManager::plugins().end(),
+    const QVector<ExtensionSystem::PluginSpec *> &specs = ExtensionSystem::PluginManager::plugins();
+    const auto pluginIt = std::find_if(specs.cbegin(), specs.cend(),
                                  [](const ExtensionSystem::PluginSpec *p) {
         return p->name() == "QmlPreview";
     });
 
-    if (pluginIt != ExtensionSystem::PluginManager::plugins().constEnd())
+    if (pluginIt != specs.cend())
         return (*pluginIt)->plugin();
 
     return nullptr;
