@@ -1,8 +1,8 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
 ** Author: Andreas Hartmetz, KDAB (andreas.hartmetz@kdab.com)
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -10,27 +10,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef LIBVALGRIND_PROTOCOL_THREADEDPARSER_H
-#define LIBVALGRIND_PROTOCOL_THREADEDPARSER_H
+#pragma once
 
 #include <QObject>
 
@@ -52,8 +46,8 @@ class ThreadedParser : public QObject
     Q_OBJECT
 
 public:
-    explicit ThreadedParser(QObject *parent = 0);
-    ~ThreadedParser();
+    explicit ThreadedParser(QObject *parent = nullptr);
+    ~ThreadedParser() override;
 
     QString errorString() const;
 
@@ -62,14 +56,13 @@ public:
     bool waitForFinished();
     bool isRunning() const;
 
-public Q_SLOTS:
     ///@warning will move @p stream to a different thread and take ownership of it
     void parse(QIODevice *stream);
 
-private Q_SLOTS:
+private:
     void slotInternalError(const QString &errorString);
 
-Q_SIGNALS:
+signals:
     void status(const Valgrind::XmlProtocol::Status &status);
     void error(const Valgrind::XmlProtocol::Error &error);
     void internalError(const QString &errorString);
@@ -84,5 +77,3 @@ private:
 
 } // XmlProtocol
 } // Valgrind
-
-#endif //LIBVALGRIND_PROTOCOL_THREADEDPARSER_H
